

drop table class_students;
drop table class_instructors;
drop table classes;
drop table catalog;
drop table student_ctas;
drop table sponsor_email_tracking;
drop table addl_users;
drop table users;
drop table addresses;
drop table domain;

create table addresses (
 address_id number(6) constraint nn_addressid NOT NULL
		      constraint pk_addressid primary key,
 address varchar2(50),
 city	varchar2(30),
 state 	varchar2(20),
 zip	varchar2(11),
 country varchar2(3),
 address2 varchar2(50),
 poc varchar2(50)
);

create table domain (
 domain_id number(6) constraint nn_domainid NOT NULL
		     constraint pk_domainid primary key,
 domain_name varchar2(30) constraint nn_dmname NOT NULL,
 order_num number(4) constraint nn_order NOT NULL,
 valid_indctr varchar2(1) constraint nn_dvalid NOT NULL,
 char_code varchar2(30),
 expsn varchar2(200) 
);


create table users (
 oracle_uid varchar(30) constraint nn_tmduid NOT NULL
			constraint pk_tmduid primary key,
 create_date date constraint nn_tmdcdate NOT NULL,
 user_status varchar2(10) constraint nn_tmdustatus NOT NULL,
 user_type varchar2(20) constraint nn_tmdutype NOT NULL,
 address_id number(6) constraint nn_tmdaddrid NOT NULL
		      constraint fk_tmdaddrid 
			references addresses(address_id),
 submitter varchar2(30),
 first_nam varchar2(35),
 last_nam varchar2(35),
 middle_nam varchar2(35),
 surtitle varchar2(7),
 purpose varchar2(2000),
 position varchar2(60),
 approved_date date,
 expiration_date date,
 citizenship varchar2(30),
 fn_indctr varchar2(1),
 business_type varchar2(25),
 business_name varchar2(50),
 permit_type varchar2(25),
 permit_num varchar2(20),
 mail_stop varchar2(20),
 room_num varchar2(20),
 phone varchar2(15),
 phone_ext varchar2(5),
 alt_phone varchar2(15),
 fax varchar2(15),
 email varchar2(50),
 poc_uid varchar2(30),
 spons_agency_uid varchar2(30),
 kdc_enabled varchar(1),
 home_msrc varchar2(7),
 kerberos_realm varchar2(30),
 sctn_3_recd_indctr varchar2(1),
 removal_date date,
 tech_indctr varchar2(1),
 tracker_indctr varchar2(1),
 organization varchar2(60),
 nac_type varchar2(10),
 ssan_last4 varchar2(4),
 user_comment varchar2(2000)
);

create table addl_users (
oracle_uid varchar2(30) constraint nn_oid NOT NULL
			constraint pk_oid primary key
			constraint fk_oid 
				references users(oracle_uid),
share_email varchar2(1),
list_email varchar2(1),
logged_in varchar2(1),
tmd_password varchar2(30),
who_disable varchar2(30),
who_enable varchar2(30),
msrc_nam varchar2(30),
login_uid varchar2(30),
nac_verified varchar2(30),
nac_check_date date
);

create table student_ctas (
 student_uid varchar2(30) constraint nn_ctauid NOT NULL
			  constraint fk_ctauid
				references users(oracle_uid),
 cta varchar2(3) constraint nn_cta NOT NULL,
 type varchar2(8) default NULL,
 constraint pk_stucta primary key (student_uid,cta)
);

create table catalog (
 course_id number(10)  constraint nn_courseid NOT NULL
		       constraint pk_courseid primary key,
 course_num varchar2(10),
 title varchar2(200) constraint nn_title NOT NULL,
 overview varchar2(4000) constraint nn_ovweview NOT NULL,
 duration number(6) constraint nn_duration NOT NULL,
 agenda varchar2(4000),
 prerequisites varchar2(200),
 class_size number(10) constraint nn_csize NOT NULL,
 catalog_type varchar(1)
);


create table classes (
 class_id number(10)  constraint nn_classid NOT NULL
		      constraint pk_classid primary key,
 course_id number(10) constraint nn_cid NOT NULL
		      constraint fk_cid 
				references catalog(course_id),
 class_start_date date constraint nn_classdate NOT NULL,
 class_size number(10) constraint nn_classsize NOT NULL,
 location varchar2(50) constraint nn_location NOT NULL,
 city varchar2(30) constraint nn_city NOT NULL,
 state varchar2(2) constraint nn_state NOT NULL,
 building varchar2(30),
 room varchar2(20),
 system_use varchar2(1),
 student_materials varchar2(1),
 tape varchar2(1),
 webhost varchar2(1),
 copresentor varchar2(1),
 class_type varchar(1),
 copresentor_names varchar2(200)
);


create table class_instructors (
 class_id number(10) constraint nn_idclass NOT NULL
	 	     constraint fk_idclass
			references classes(class_id),
 instructor_uid varchar2(30) constraint nn_insuid NOT NULL
			     constraint fk_insuid 
				references users(oracle_uid),
 constraint pk_idclass primary key (class_id,instructor_uid)
);


create table class_students (
 class_id number(10) constraint nn_stuclassid NOT NULL
		     constraint fk_stuclassid 
			references classes(class_id),
 student_uid varchar2(30) constraint nn_stuuid NOT NULL
 			  constraint fk_stuuid 
				references users(oracle_uid),
 register_date date constraint nn_regdate NOT NULL,
 confirm_indctr varchar2(1) constraint nn_confirm NOT NULL,
 constraint pk_stuclassid primary key (class_id,student_uid)
);

create table sponsor_email_tracking (
 oracle_uid varchar2(30) constraint nn_emailid NOT NULL
			 constraint fk_emailid 
				references users(oracle_uid),
 date_sent date constraint nn_sentdate NOT NULL,
 sponsor_uid varchar2(30) constraint nn_sponuid NOT NULL,
 accept varchar2(1) constraint nn_accept NOT NULL,
 modified_by varchar2(30) constraint nn_by NOT NULL,
 modified_date date constraint nn_mdate NOT NULL,
 constraint pk_spon primary key (oracle_uid,date_sent)
);

drop table user_emails;
create table user_emails (
 oracle_uid varchar2(30) constraint nn_uemailid NOT NULL
			 constraint fk_uemailid 
				references users(oracle_uid),
 sent_date date constraint nn_usentdate NOT NULL,
 sender varchar2(30) constraint nn_usender NOT NULL,
 email_type number(2) constraint nn_uemailtype NOT NULL
);

drop view user_info;
create view user_info as
	select a.*,b.share_email,b.list_email,b.logged_in,b.tmd_password,
		b.msrc_nam,b.who_enable, b.who_disable,b.login_uid,
		b.nac_check_date,b.nac_verified,
		c.address,c.city,c.state,c.zip,c.country,c.address2,c.poc 
	from users a, addl_users b,addresses c
 	where a.oracle_uid = b.oracle_uid
	and a.address_id = c.address_id;

drop view class_schedule;
create view class_schedule as
	select a.agenda, a.duration, a.prerequisites,a.title,a.course_num,
		a.overview,a.catalog_type,
		b.class_id,b.class_start_date,b.location,b.class_type,
		b.city,b.state,b.building,b.room,b.course_id,b.class_size,
		c.instructor_uid,
		d.first_nam, d.last_nam
	from catalog a, classes b, class_instructors c, users d
	where a.course_id = b.course_id
	and   b.class_id = c.class_id
	and   c.instructor_uid = d.oracle_uid;

drop view class_roster;
create view class_roster as
	select a.class_id, b.phone, b.phone_ext,
		b.first_nam, b.last_nam, b.email, c.share_email
	from class_students a, users b, addl_users c
	where a.student_uid = b.oracle_uid
	and a.student_uid = c.oracle_uid;
	
