#
# Example for building demo OCI programs:
#
# 1. All OCI demos:
#
#    make -f demo_rdbms.mk demos
#
# 2. A single OCI demo:
#
#    make -f demo_rdbms.mk build EXE=demo OBJS="demo.o ..."
#    e.g. make -f demo_rdbms.mk build EXE=oci02 OBJS=oci02.o
#
# 3. A single OCI demo with static libraries:
#
#    make -f demo_rdbms.mk build_static EXE=demo OBJS="demo.o ..."
#    e.g. make -f demo_rdbms.mk build_static EXE=oci02 OBJS=oci02.o
#
# 4. To re-generate shared library:
#
#    make -f demo_rdbms.mk generate_sharedlib
#
#
# Example for building demo DIRECT PATH API programs:
#
# 1. All DIRECT PATH API demos:
#
#    make -f demo_rdbms.mk demos_dp
#
# 2. A single DIRECT PATH API demo:
#
#    make -f demo_rdbms.mk build_dp EXE=demo OBJS="demo.o ..."
#    e.g. make -f demo_rdbms.mk build_dp EXE=cdemodp_lip OBJS=cdemodp_lip.o
#
#
# Example for building external procedures demo programs:
#
# 1. All external procedure demos:
#
# 2. A single external procedure demo whose 3GL routines do not use the 
#    "with context" argument:
#
#    make -f demo_rdbms.mk extproc_no_context SHARED_LIBNAME=libname 
#                                             OBJS="demo.o ..."
#    e.g. make -f demo_rdbms.mk extproc_no_context SHARED_LIBNAME=epdemo.so
#                                             OBJS="epdemo1.o epdemo2.o"
#
# 3. A single external procedure demo where one or more 3GL routines use the 
#    "with context" argument:
#
#    make -f demo_rdbms.mk extproc_with_context SHARED_LIBNAME=libname 
#                                             OBJS="demo.o ..."
#    e.g. make -f demo_rdbms.mk extproc_with_context SHARED_LIBNAME=epdemo.so
#                                             OBJS="epdemo1.o epdemo2.o"
#    e.g. make -f demo_rdbms.mk extproc_with_context 
#                       SHARED_LIBNAME=extdemo2.so OBJS="extdemo2.o"
#
# 4. To link C++ demos:
#
#    make -f demo_rdbms.mk c++demos
#
#
# NOTE: 1. ORACLE_HOME must be either:
#		   . set in the user's environment
#		   . passed in on the command line
#		   . defined in a modified version of this makefile
#
#       2. If the target platform support shared libraries (e.g. Solaris)
#          look in the platform specific documentation for information
#          about environment variables that need to be properly
#          defined (e.g. LD_LIBRARY_PATH in Solaris).
#


#include $(ORACLE_HOME)/rdbms/lib/env_rdbms.mk
include  env_rdbms.mk 

RDBMSLIB=$(ORACLE_HOME)/rdbms/lib/

LDFLAGS=-L$(LIBHOME) -L$(ORACLE_HOME)/rdbms/lib
LLIBPSO=`cat $(ORACLE_HOME)/rdbms/lib/psoliblist`

FC=f77
COB=cob
COBFLAGS=-C IBMCOMP -x
COBGNTFLAGS=-C IBMCOMP -u
CPLPL=CC

#CC=/opt/SUNWspro4.2/SUNWspro/bin/cc

CC=/usr/local/bin/gcc

# directory that contain oratypes.h and other oci demo program header files
INCLUDE= -I$(ORACLE_HOME)/rdbms/demo -I$(ORACLE_HOME)/rdbms/public -I$(ORACLE_HOME)/plsql/public -I$(ORACLE_HOME)/network/public

#
CONFIG = $(ORACLE_HOME)/rdbms/lib/config.o

# module to be used for linking with non-deferred option

# flag for linking with non-deferred option (default is deferred mode)
NONDEFER=false

# libraries for linking oci programs
OCISHAREDLIBS=$(TTLIBS) $(LLIBTHREAD)
OCISTATICLIBS=$(STATICTTLIBS) $(LLIBTHREAD)

PSOLIBLIST=$(ORACLE_HOME)/rdbms/lib/psoliblist
CLEANPSO=rm -f $(PSOLIBLIST); $(GENPSOLIB)

DOLIB=$(ORACLE_HOME)/lib/liborcaccel.a
DUMSDOLIB=$(ORACLE_HOME)/lib/liborcaccel_stub.a
REALSDOLIB=/usr/lpp/orcaccel/liborcaccel.a


PROC=$(ORACLE_HOME)/bin/proc
PCCINCLUDE= include=$(ORACLE_HOME)/precomp/public
PCCI=-I$(ORACLE_HOME)/precomp/public
#USERID=scott/tiger
USERID=sccs/sccs
PCCPLSFLAGS= $(PCCINCLUDE) ireclen=132 oreclen=132 sqlcheck=full \
ltype=none user=$(USERID)
LLIBSQL= -lsql
PROLDLIBS= $(LLIBSQL) $(TTLIBS)

DEMO_MAKEFILE = demo_rdbms.mk
 
DEMOS = wowstub
DEMOS_DP = cdemodp_lip

C++DEMOS = cdemo6

WOWOBJS = oci.o util.o

.SUFFIXES: .o .cob .for .c


demos: $(DEMOS) extdemo2

demos_dp: $(DEMOS_DP) 

generate_sharedlib:
	$(SILENT)$(ECHO) "Building client shared library ..."
	$(SILENT)$(ECHO) "Calling script $$ORACLE_HOME/bin/genclntsh ..."
	$(GENCLNTSH)
	$(SILENT)$(ECHO) "The library is $$ORACLE_HOME/lib/libclntsh.so... DONE"

MAKECPLPLDEMO= \
	@if [ "$(NONDEFER)" = "true" -o "$(NONDEFER)" = "TRUE" ] ; then \
	$(ECHODO) $(CPLPL) $(LDFLAGS) -o $(EXE) $? $(NDFOPT) $(OCISHAREDLIBS); \
	else \
	$(ECHODO) $(CPLPL) $(LDFLAGS) -o $(EXE) $? $(OCISHAREDLIBS); \
	fi

$(DEMOS):
	$(MAKE) -f $(DEMO_MAKEFILE) build EXE=$@ OBJS=$@.o

$(DEMOS_DP): cdemodp.c cdemodp0.h cdemodp.h
	$(MAKE) -f $(DEMO_MAKEFILE) build_dp EXE=$@ OBJS=$@.o

c++demos: $(C++DEMOS)

$(C++DEMOS):
	$(MAKE) -f $(DEMO_MAKEFILE) buildc++ EXE=$@ OBJS=$@.o

buildc++: $(OBJS) $(WOWOBJS)
	$(MAKECPLPLDEMO)

.cc.o:
	$(CPLPL) -c $(INCLUDE) $<

build: $(LIBCLNTSH) $(OBJS) $(WOWOBJS)
	$(ECHODO) $(CC) $(LDFLAGS) -o $(EXE) ${WOWOBJS} $(OBJS) $(OCISHAREDLIBS)

extdemo2:
	$(MAKE) -f $(DEMO_MAKEFILE) extproc_with_context SHARED_LIBNAME=extdemo2.so OBJS="extdemo2.o"

.c.o:
	$(ECHODO) $(CC) -c $(INCLUDE) $<

build_dp: $(LIBCLNTSH) $(OBJS) cdemodp.o
	   $(ECHODO) $(CC) $(LDFLAGS) -o $(EXE) cdemodp.o $(OBJS) $(OCISHAREDLIBS)

build_static: $(OBJS)
	$(ECHODO) $(CC) $(LDFLAGS) -o $(EXE) $(OBJS) $(SSDBED) $(DEF_ON) \
	$(OCISTATICLIBS)

extproc_no_context: $(OBJS)
	$(LD) $(SHARED_LDFLAG) $(SHARED_LIBNAME) $(OBJS)

extproc_with_context: $(OBJS) $(LIBCLNTSH)
	$(LD) $(SHARED_LDFLAG) $(SHARED_LIBNAME) $(OBJS) $(OCISHAREDLIBS) \
	$(LIBEXTP)

# these two targets are the same as the ones above extproc_no_context and 
# extproc_with_context. They are preserved for backward compatibility.
extproc_nocallback: $(OBJS)
	$(LD) $(SHARED_LDFLAG) $(SHARED_LIBNAME) $(OBJS)

extproc_callback: $(OBJS) $(LIBCLNTSH)
	$(LD) $(SHARED_LDFLAG) $(SHARED_LIBNAME) $(OBJS) $(OCISHAREDLIBS) \
	$(LIBEXTP)

clean:
	$(RM) -f $(DEMOS) extdemo2 *.o *.so



