/* Copyright (c) 1994 by M.Lonnroth, Oracle Corp.  All Rights Reserved */

/***
   NAME
     oci
   PURPOSE
     Wrappers for OCI 
   NOTES
     
   HISTORY
     mloennro - Oct 13, 1994: Created.
     $Log: oci.c,v $
 * Revision 1.1  1994/12/28  13:45:38  mloennro
 * Initial revision
 *
 * Revision 1.1  1994/10/17  20:52:06  mloennro
 * Initial revision
 *
***/

static char rcsid[] = "$Header: /home/mloennro/sdk/cwow/RCS/oci.c,v 1.1 1994/12/28 13:45:38 mloennro Exp mloennro $";

#include "oci.h"
#ifdef DEBUG
extern int debuglevel;
#endif

char errbuf[128];

/* Display error and die */
#ifdef __STDC__
void dbErr (char *src, Lda_Def *lda, Cda_Def *cda)
#else
void dbErr(src,lda,cda)
     char *src;
     Lda_Def *lda;
     Cda_Def *cda;
#endif
{
     text buf[2000];

     fprintf(stderr,
	     "\n\n*** Error ***\nFile: %s\nFunction: %s: OCI error %d\n\n", 
	     rcsid,src,(int)cda->rc );
     (void)oerhms(lda,cda->rc,buf,sizeof(buf));
     RAISE_ERROR(buf);
}

/* Logon to Oracle */
#ifdef __STDC__
void dbLogon (dbConnection *db,char *uid )
#else
void dbLogon(db,uid)
     dbConnection *db;
     char *uid;
#endif
{
#ifdef DEBUG
     if (debuglevel >= 1)
       (void) fprintf(stderr, "DEBUG: dbLogon(%s) ",uid);
#endif /* DEBUG */

     if ((db->lda=(Lda_Def*)malloc(sizeof(Lda_Def)))==NULL)
       {
	    (void)strcpy(errbuf,"dbLogon(): malloc() failed.\n");
	    fprintf(stderr,errbuf);
	    RAISE_ERROR(errbuf);
       }
 
     if (orlon(db->lda,db->hda,uid,-1,(char*)0,-1,-1))
       {
	    /* (void)sprintf(errbuf,"dbLogon(): orlon() returns %ud.\n",
			  db->lda->rc);*/
	   (void)sprintf(errbuf,
	    "The AskNPAC Oracle Database Server is Down. Please send an email to Gang at <a href=\"mailto:gcheng@npac.syr.edu\">gcheng@npac.syr.edu</a>.<p>\n");
	    fprintf(stderr,errbuf);
	    (void)sprintf(errbuf,
			  "If you are participating an AskNPAC Chat-Board session, <br>");
	    fprintf(stderr,errbuf);
	    (void)sprintf(errbuf,
"you may continue the chat on <a href=\"http://kayak.npac.syr.edu:1963/cgi-bin/chat/chat.html\">this backup server</a>.\n");
	    fprintf(stderr,errbuf);
	    RAISE_ERROR(errbuf);
       }
#ifdef DEBUG
     if (debuglevel >= 1)
       (void) fprintf(stderr, "OK\n");
#endif /* DEBUG */
}

/* Logoff with error check */
#ifdef __STDC__
void dbLogoff (Lda_Def *lda)
#else
void dbLogoff(lda)
     Lda_Def *lda;
#endif
{
     if (ologof(lda))
       dbErr("ologof() in dbLogoff()",lda,lda);
}

/* Allocate, open, parse a cursor */
#ifdef __STDC__
Cda_Def *dbParse (Lda_Def *lda,char *sql)
#else
Cda_Def *dbParse(lda,sql)
     Lda_Def *lda;
     char *sql;
#endif
{
     Cda_Def *cda;
#ifdef DEBUG
     if (debuglevel >= 1)
       (void) fprintf(stderr, "DEBUG: dbParse(\"%s\")\n",sql);
#endif /* DEBUG */     
     if ((cda=(Cda_Def*)malloc(sizeof(Cda_Def))) == (Cda_Def*)NULL)
       {
	    /* Panic */
	    (void)orol(lda);
	    (void)ologof(lda);
	    wowFail("dbParse(): malloc() error.\n");
       }
     
     if (oopen(cda,lda,(text*)0,(sword)-1,(sword)-1,(text*)0,(sword)-1))
       {
	    dbErr ("oopen() in dbParse()",lda,cda);
       }
     
     if (oparse(cda,sql,-1,0,2))
       {
	    dbErr("oparse() in dbParse()",lda,cda);
       }

     return cda;
}

/* Bind 1 char[] variable */
#ifdef __STDC__
void dbBind (Lda_Def *lda,Cda_Def *cda,text *sqlvar,text *progv,sb2 *indp)
#else
void dbBind(lda,cda,sqlvar,progv,indp)
     Lda_Def *lda;
     Cda_Def *cda;
     text *sqlvar;
     text *progv;
     sb2 *indp;
#endif
{
#ifdef DEBUG
     if (debuglevel >= 1)
       (void) fprintf(stderr, "DEBUG: dbBind(%s)\n",sqlvar);
#endif /* DEBUG */

     if (obndrv(cda,sqlvar,-1,(ub1*)progv,255,OCI_CHARZ,-1,indp,
		(text*)0,-1,-1))
       {
	    dbErr("obndrv() in dbBind()",lda,cda);
       }
}

/* Bind 1 int variable */
#ifdef __STDC__
void dbBindNum (Lda_Def *lda,Cda_Def *cda,text *sqlvar,int *progv,sb2 *indp)
#else
void dbBindNum(lda,cda,sqlvar,progv,indp)
     Lda_Def *lda;
     Cda_Def *cda;
     text *sqlvar;
     text *progv;
     sb2 *indp;
#endif
{
#ifdef DEBUG
     if (debuglevel >= 1)
       (void) fprintf(stderr, "DEBUG: dbBindNum(%s)\n",sqlvar);
#endif /* DEBUG */

     if (obndrv(cda,sqlvar,-1,(ub1*)progv,sizeof(int),OCI_INT,-1,indp,
		(text*)0,-1,-1))
       {
	    dbErr("obndrv() in dbBindNum()",lda,cda);
       }
}

/* Bind one char array variable */
#ifdef __STDC__
void dbBindArr (Lda_Def *lda,Cda_Def *cda,text *sqlvar,ub4 arrsz,text **progv,
		ub2 *progl,sb2 *indp,ub2 *rcode,ub4 *retarr)
#else
void dbBindArr(lda,cda,sqlvar,arrsz,progv,progl,indp,rcode,retarr)
     Lda_Def *lda;
     Cda_Def *cda;
     text *sqlvar;
     ub4 arrsz;
     text *progv;
     ub2 *progl;
     sb2 *indp;
     ub2 *rcode;
     ub4 *retarr;
#endif
{
#ifdef DEBUG
     if (debuglevel >= 1)
       (void) fprintf(stderr, "DEBUG: dbBindArr(%s)\n",sqlvar);
#endif /* DEBUG */

     if (obndra(cda,sqlvar,-1,(ub1*)progv,255,OCI_CHAR,-1,indp,progl,
		rcode,arrsz,retarr,(text*)0,-1,-1))
       {
	    dbErr("obndrv() in dbBindArr()",lda,cda);
       }
}

/* Execute cursor */
#ifdef __STDC__
void dbExe (Lda_Def *lda,Cda_Def *cda)
#else
void dbExe(lda,cda)
     Lda_Def *lda;
     Cda_Def *cda;
#endif
{
#ifdef DEBUG
     if (debuglevel >= 1)
       (void) fprintf(stderr, "DEBUG: dbExe()\n");
#endif /* DEBUG */

     if (oexec(cda))
       dbErr("oexec() in dbExe()",lda,cda);
}

/* End transaction and logoff */
#ifdef __STDC__
void dbFinish (Lda_Def *lda)
#else
void dbFinish(lda)
     Lda_Def *lda;
#endif
{
     if (lda->rc)
       (void)orol(lda);
     else
       (void)ologof(lda);

     free(lda);
}

/* Close cursor */
#ifdef __STDC__
void dbClose (Lda_Def *lda,Cda_Def *cda)
#else
void dbClose(lda,cda)
     Lda_Def *lda;
     Cda_Def *cda;
#endif
{
     if (oclose(cda))
       dbErr("oclose() in dbClose()",lda,cda);
}
