#
# Example for linking demo OCI programs:
#
# deferred mode (default):
#	make -f oracle.mk cdemo1
#
# non-deferred mode:
#	make -f oracle.mk NONDEFER=true cdemo1
#
#
# NOTE:   ORACLE_HOME must be either:
#		. set in the user's environment
#		. passed in on the command line
#		. defined in a modified version of this makefile
#
# Following macro contains ORACLE libraries that are linked to create:
# OCILDLIBS	: OCI application executables
#

include env_rdbms.mk

LDFLAGS=-L$(LIBHOME) -L$(ORACLE_HOME)/rdbms/lib
ECHO=$(ORACLE_HOME)/bin/echodo
LLIBPSO=`cat $(ORACLE_HOME)/rdbms/lib/psoliblist`
 
FC=f77
COB=cob
COBFLAGS=-C IBMCOMP -x
COBGNTFLAGS=-C IBMCOMP -u
CPLPL=CC

LLAIO=

# directory that contain oratypes.h and other oci demo program header files
INCLUDE= -I$(ORACLE_HOME)/rdbms/demo -I$(ORACLE_HOME)/rdbms/public -I.

#
CONFIG = $(ORACLE_HOME)/rdbms/lib/config.o

# module to be used for linking with non-deferred option
NDFOPT= $(ORACLE_HOME)/rdbms/lib/kpundf.o

# flag for linking with non-deferred option (default is deferred mode)
NONDEFER=false

# libraries for linking oci programs
OCILDLIBS= $(LIBCLIENT) $(DEVTTLIBS) $(LLIBCORE)

# object for linking oci programs in Fortran
OCIFOR=$(ORACLE_HOME)/rdbms/lib/ocifor.o
 
# object for linking oci programs in Cobol
OCICOB=$(ORACLE_HOME)/rdbms/lib/ocicob.o

PSOLIBLIST=$(ORACLE_HOME)/rdbms/lib/psoliblist
CLEANPSO=rm -f $(PSOLIBLIST); $(GENPSOLIB)


DEMO_MAKEFILE = oracle.mk
 
DEMOS = wowstub
COBDEMOS = cbdem1 cbdem2 cbdem3 
FORDEMOS = fdemo2 fdemo3
CPLPLDEMOS = cdemo6


WOWOBJS = oci.o util.o

.SUFFIXES: .o .cob .for

MAKEDEMO= \
	@if [ "$(NONDEFER)" = "true" -o "$(NONDEFER)" = "TRUE" ] ; then \
	$(ECHO) $(CC) $(LDFLAGS) -o $(EXE) $? $(NDFOPT) $(OCILDLIBS) $(CLIBS); \
	else \
	$(ECHO) $(CC) $(LDFLAGS) -o $(EXE) $? $(OCILDLIBS) $(CLIBS); \
	fi

demos: $(DEMOS)

$(DEMOS):
	$(MAKE) -f $(DEMO_MAKEFILE) build EXE=$@ OBJS=$@.o

build: $(OBJS) $(WOWOBJS)
	$(MAKEDEMO)


MAKECOBDEMO= \
	@if [ "$(NONDEFER)" = "true" -o "$(NONDEFER)" = "TRUE" ] ; then \
	$(ECHO) $(COB) $(COBFLAGS) $(LDFLAGS) -o $(EXE) $? $(NDFOPT) $(OCICOB) $(OCILDLIBS); \
	else \
	$(ECHO) $(COB) $(COBFLAGS) $(LDFLAGS) -o $(EXE) $? $(OCICOB) $(OCILDLIBS); \
	fi


cobdemos: $(COBDEMOS)
 
$(COBDEMOS):
	$(MAKE) -f $(DEMO_MAKEFILE) buildcob EXE=$@ OBJS=$@.cob
 
buildcob: $(OBJS)
	$(MAKECOBDEMO)
 


MAKEFORDEMO= \
	@if [ "$(NONDEFER)" = "true" -o "$(NONDEFER)" = "TRUE" ] ; then \
	$(ECHO) $(FC)  $(LDFLAGS) -o $(EXE) $? $(NDFOPT) $(OCIFOR) $(OCILDLIBS); \
	else \
	$(ECHO) $(FC) $(LDFLAGS) -o $(EXE) $? $(OCIFOR) $(OCILDLIBS); \
	fi

fordemos:  $(FORDEMOS)
 
$(FORDEMOS):
	$(MAKE) -f $(DEMO_MAKEFILE) buildfor EXE=$@ OBJS=$@.o
 
buildfor:  $(OBJS)
	$(MAKEFORDEMO)

MAKECPLPLDEMO= \
	@if [ "$(NONDEFER)" = "true" -o "$(NONDEFER)" = "TRUE" ] ; then \
	$(ECHO) $(CPLPL) $(LDFLAGS) -o $(EXE) $? $(NDFOPT) $(OCILDLIB); \
	else \
	$(ECHO) $(CPLPL) $(LDFLAGS) -o $(EXE) $? $(OCILDLIBS); \
	fi

c++demos: $(CPLPLDEMOS)

$(CPLPLDEMOS):
	$(MAKE) -f $(DEMO_MAKEFILE) buildc++ EXE=$@ OBJS=$@.o

buildc++: $(OBJS)
	$(MAKECPLPLDEMO)


.for.o:
	$(FC) -c $(FCFLAGS) $<

.cc.o:
	$(CPLPL) -c $(INCLUDE) $<


clean: cleanexecs cleanobjs

cleanobjs:
	$(REMOVE) *.o
	$(REMOVE) *.int

cleanexecs:
	$(REMOVE) $(DEMOS) $(FORDEMOS) $(COBDEMOS) $(C++DEMOS)

 


