/**
* @author  Umesh Kulkarni
* @version 1.0
*
* Development Environment         : JDeveloper 2.0
*
* Name of the Application         : BFILEFrame.java
*
* Creation/ Modification History  :
*
*       ukulkarn.in       11-Jan-1999    Creation
*
* Overview of the Application     :
*
*  This class handles the GUI for the BFILESample application.
*
**/

import java.awt.*;
import java.awt.event.*;
import java.io.*;  // Package for handling IOStreams

/** Use the following import statements with SWING version 1.0.2 **/
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;

/** Use the following import statements with SWING version 1.1 **/
// import javax.swing.*;
// import javax.swing.event.*;

public class BFILEFrame extends JFrame {

  BFILESample  m_parent;   // The Main Application Class which uses this frame

  // Labels used in the UI
  JLabel m_bfileImageLabel = new JLabel();

  //Displays error and status messages
  JTextField m_statusField = new JTextField();

  //Buttons for specifying "SELECT" and "EXIT" operations
  JButton m_exitButton = new JButton();
  JButton m_selectButton = new JButton();

  // Objects for handling and displaying image
  Image m_image = null;  // Image Object
  MediaTracker m_tracker = null; // Media Tracker
  Panel m_panel = new Panel(); // Panel to Show Image Read from BFILE_TABLE

  GridBagLayout m_gridBagLayout = new GridBagLayout();// Layout Manager

  /**
  *  Constructor. Initializes GUI components and sets up the Listeners
  **/
  public BFILEFrame(BFILESample p_parent){
    try {
      m_parent = p_parent;
      jbInit();  // Initialize all the GUI Components
      setupListeners(); // Setting up actionlisteners for GUI components
    }catch (Exception e) {
      putStatus(" Error while initializing ..." + e.toString());
    }
  }

  /**
  * Method to Prompt the user for Full Directory Path, where he/she has copied
  * the image file (architect.gif).
  * After user enters the Full Directory Path of the image file, it is returned
  * back to the main application as a String.
  **/
  public String getDirectoryPath(){
    // Setup the JOptionPane for accepting Full Directory Path

    JTextArea l_dirTextArea = new JTextArea(1,30);
    l_dirTextArea.setText("/private/otn/image");// Setting Default Initial Value of dirPath
    Object[] l_objects = new Object[2];
    l_objects[0] = "Enter Full Directory Path of image file on Database Server";
    l_objects[1] = l_dirTextArea;
    String[] l_options = {"Proceed","Cancel & Exit"};
    // If user has entered directory path and presses Proceed then return
    // directory path else return 'Cancel' String
    if (JOptionPane.showOptionDialog(
        this, l_objects,"Enter Full Directory Path", JOptionPane.DEFAULT_OPTION,
        JOptionPane.QUESTION_MESSAGE, null, l_options, l_options[0]) == 0)
          return l_dirTextArea.getText();
    else
       return "Cancel";
  }

  /**
  * Method to Display the image fetched from the BFILE Column.
  * Note that the parameter passed is Binary Input Stream
  **/
  public void displayFile(InputStream p_instream) {
    try {
      // Clearing Contents on the image Panel
      Graphics l_graphics = m_panel.getGraphics();
      Rectangle l_rect = m_panel.getBounds();
      l_graphics.clearRect(80,50,l_rect.getSize().width,l_rect.getSize().height);

      // Open File Output Stream to store the Fetched image file locally as bfile.gif
      FileOutputStream l_file = new FileOutputStream("bfile.gif");

      // Read the Binary File in chunks of 10 bytes
      int l_chunkSize = 10;
      long l_lengthRead;
      byte[] l_buffer = new byte[l_chunkSize];

      while ((l_lengthRead = p_instream.read(l_buffer)) != -1) {
        l_file.write(l_buffer);
        l_file.flush();
      }
      p_instream.close(); // Close the input Stream
      l_file.close(); // Close the File Output Stream

      // Display the image Stored in the file bfile.gif
      m_tracker = new MediaTracker(this); //

      // Get the Fetched Image from bfile.gif
      m_image = Toolkit.getDefaultToolkit().getImage("bfile.gif");
      m_tracker.addImage(m_image,0);
      m_tracker.waitForID(0);
      l_graphics = m_panel.getGraphics(); // Get the Graphics Object
      l_graphics.drawImage(m_image,80,50,this); // Draw Image

      appendStatus("Image from BFILE displayed");
    } catch (IOException ex) { // Trap IO Errors
         putStatus(" IOErrors while displaying Image File " + ex.toString());
    }
      catch (InterruptedException ex) { // Trap Interrupted Errors
         putStatus(" Stream Interruption Errors while displaying Image File\n "
                     + ex.toString());
    }
  }

  /**
  *  Sets up the listeners that respond to user actions, like button
  *  press.
  **/
  public void setupListeners() {
    // Setup the exit button listener
    m_exitButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // Pass the Event to Parent Class for handling
        m_parent.dispatchEvent("EXIT");
      }
     });

   // Setup the Select button listener
   m_selectButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // Pass the Event to Parent Class for handling
        m_parent.dispatchEvent("SELECT");
      }
     });

   // The window exit listener
   this.addWindowListener(new java.awt.event.WindowAdapter() {
     public void windowClosing(WindowEvent e) {
        m_parent.dispatchEvent("EXIT");
     }
    });
  }

  /**
   Status field message display handlers
  **/
      public void clearStatus() {         // Clears status field
        m_statusField.setText("");
       }
      public void putStatus(String p_status) { // Puts p_status on status field
        m_statusField.setText(p_status);
        m_statusField.setScrollOffset(0);
      }
      public void appendStatus(String p_status) {  //Appends p_status to status
                                                   //Field
        String l_status = m_statusField.getText();
        m_statusField.setText(l_status+" "+p_status);
        m_statusField.setScrollOffset(0);
      }


  /**
  * Initialize all the GUI components and the layout
  **/
  public void jbInit() throws Exception {
    this.getContentPane().setLayout(m_gridBagLayout);
    m_panel.setBackground(Color.black);
    m_exitButton.setLabel("EXIT");
    m_selectButton.setLabel("SELECT");
    m_statusField.setFont(new Font("Monospaced",0,12));
    m_statusField.setBackground(Color.lightGray);
    m_statusField.setDisabledTextColor(Color.lightGray);
    m_statusField.setEditable(false);
    m_bfileImageLabel.setText("BFILE Image");
    this.getContentPane().add(m_panel,
         new GridBagConstraints2(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.SOUTH,
         GridBagConstraints.NONE,  new Insets(0, 12, 3, 9), 404, 195));
    this.getContentPane().add(m_exitButton,
         new GridBagConstraints2(1, 2, 1, 1, 0.0, 0.0 ,GridBagConstraints.SOUTHEAST,
         GridBagConstraints.NONE, new Insets(32, 14, 11, 4), 45, 9));
    this.getContentPane().add(m_selectButton,
         new GridBagConstraints2(0, 2, 1, 1, 0.0, 0.0,GridBagConstraints.SOUTHEAST,
         GridBagConstraints.NONE, new Insets(21, 212, 11, 3), 61, 8));
    this.getContentPane().add(m_statusField,
         new GridBagConstraints2(0, 3, 3, 1, 1.0, 0.0 ,GridBagConstraints.WEST,
         GridBagConstraints.NONE, new Insets(3, 2, 3, 2), 633, 20));
    this.getContentPane().add(m_bfileImageLabel,
         new GridBagConstraints2(0, 0, 1, 1, 0.0, 0.0,GridBagConstraints.SOUTHWEST,
         GridBagConstraints.NONE, new Insets(20, 18, 5, 0), 169, 10));
    this.setTitle("Illustrating BFILES Using JDBC");
    this.setSize(new Dimension(577, 394));
    this.setVisible(true);
  }
}






