/**
 * @author  srangan.in
 * @version 1.0
 *
 * Development Environment        :  JDeveloper 2.0
 * Name of the Application        :  LOBFrame.java
 * Creation/Modification History  :
 *
 *    srangan.in       10-JAN-1998      Created
 *
 * Overview of Application        :
 *
 * GUI for LOB Sample
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** Use the following import statements with SWING version 1.0.2**/
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.border.*;

/** Use the following import statements with SWING version 1.1**/
//import javax.swing.*;
//import javax.swing.event.*;
//import javax.swing.table.*;
//import javax.swing.border.*;

public class LOBFrame extends JFrame implements Runnable{

  LOBSample m_parent; // The sample main class, to which events are passed

  // The worker thread for processing an event using a method in the main class
  Thread m_processThread;

  // Event name to be passed to dispatchEvent in LOBSample. Used by the run
  // method, to pass the current event under process
  String m_event = null;

  //JTable related objects
  JTable        m_table;         // The JTable which displays the database rows
  genTableModel m_tableModel;     // Table Model for the JTable
  JScrollPane   m_scrollPane;     // The scroll pane holding m_jtable

  // Other GUI components
  JButton m_loadButton, m_addButton, m_exitButton;
  JPanel m_lobPanel = new JPanel();   // Hold m_tabPane (Used to give a Title)
  JTabbedPane m_tabPane = new JTabbedPane(); // Contains two tabs, m_imagePane
                                             // and m_suggestionPane
  JScrollPane m_imagePane, m_suggestionPane; // Scroll panes for map image and
                                             // suggestion text areas
  JTextArea m_sugArea = new JTextArea();     // Suggestion Text Area (CLOB col)
  JLabel m_imageArea = new JLabel();         // Airport Map image (BLOB col)

  JTextField m_statusField = new JTextField(); // Status bar

  // Layout Managers
  BorderLayout m_borderLayout = new BorderLayout(); // For m_lobPanel
// For m_contentPane
  GridBagLayout m_gridBagLayout = new GridBagLayout();

  /**
  *  Constructor. Initializes the JTable and other GUI components.
  **/
  public LOBFrame(LOBSample p_parent) {
    m_parent = p_parent;
    try {
      tableInit();  // Initialize Table Model and setup row selection listener
      jbInit();     // Initialize other GUI components
      setupListeners(); // Setup the button listeners.
    } catch (Exception ex) {
      putStatus("Error in GUI initialization");
      appendStatus(ex.toString());
    }
  }

  /**
  * Clear the map display area
  **/
  void clearMap() {
    m_imageArea = new JLabel();
    m_imagePane.setViewportView(m_imageArea);
    m_imagePane.validate();
  }

  /**
  *  This method inserts a new row into the JTable
  **/
  public void addToJTable(String p_code,String p_desc,String p_city) {
    Vector l_newrow = new Vector();

    //The column values are added to the Vector
    l_newrow.addElement(p_code);
    l_newrow.addElement(p_desc);
    l_newrow.addElement(p_city);

    //Add the row to the JTable
    m_tableModel.insertRow(l_newrow);
    m_scrollPane.repaint();
  }

  /**
  * Create a label from the image filepassed, add the label to the image area
  **/
  public void drawMap(String p_fileName) {
    ImageIcon l_mapImage = new ImageIcon(p_fileName+".gif");
    m_imageArea = new JLabel(l_mapImage);
    m_imagePane.setViewportView(m_imageArea);
    m_imagePane.validate();
    m_loadButton.setEnabled(false);
    m_addButton.setEnabled(true);
  }

  /**
  * Receives suggestion text from user using a OptioPane
  **/
  public String getSuggestionText() {
    // Setup the JOptionPane for accepting suggestions
	  JTextArea l_sugTextArea = new JTextArea(3,30); // TextArea for entering Suggestions
    Object[]   l_objects = new Object[2];  // Objects in the OptionPane
    l_objects[0] = "Please enter suggestions."; // Label
    l_objects[1] = l_sugTextArea; // Suggestions TextArea
    String[] l_options = { "Cancel", "Save"}; // Option Pane Buttons

    // Show the OptionPane, and if the save button was pressed, continue
    // processing
    if (JOptionPane.showOptionDialog(this, l_objects, "Add Suggestions",
                      JOptionPane.DEFAULT_OPTION,JOptionPane.QUESTION_MESSAGE,
                      null, l_options, l_options[1])==1)
      return l_sugTextArea.getText();
    else
      return "CANCEL";
  }

  /**
  * Get the airport code of the selected airport
  **/
  public String getSelectedCode() {
    int l_selectedRow = m_table.getSelectedRow(); // Selected row number
    return (String) m_tableModel.getRow(l_selectedRow).elementAt(0);
  }

  /**
  * Method to start a thread for event processing
  **/
  public void processEvent(String p_event) {
    m_event = new String(p_event);
    if(m_processThread == null) {
       m_processThread = new Thread(this,"ProcessThread");
       m_processThread.start();
    }
  }

  /**
  * Passes on the GUI event to dispatchEvent, where the appropriate
  * method is invoked and JDBC operations performed.
  **/
  public void run() {

    this.setCursor(Cursor.WAIT_CURSOR);
    m_parent.dispatchEvent(m_event);
    m_processThread = null;
    System.gc();
    this.setCursor(Cursor.DEFAULT_CURSOR);
  }

  /**
  *  JTable initialization. Sets up the column name and the table model
  *  for the JTable
  **/
  public void tableInit(){
    String[] l_colHeadings = {"Airport Code", "Description", "City"};
                                             // Column Headings
    Object[] l_defaultValues = {"", "", ""}; // Default Values

    // Create a table, based on a tableModel with the above structure
    m_tableModel = new genTableModel(l_colHeadings,l_defaultValues, 0);
    m_table = new JTable(m_tableModel);

    // Setup the row selection listener for the JTable
    // Allow only single selection on table
    m_table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    m_table.getSelectionModel().addListSelectionListener(
          new ListSelectionListener() {
             public void valueChanged(ListSelectionEvent e) {
              m_addButton.setEnabled(false);
              m_loadButton.setEnabled(false);
              clearMap();
              m_sugArea.setText(" ");
              ListSelectionModel lsm = (ListSelectionModel)e.getSource();
              if (!lsm.isSelectionEmpty()) {
                processEvent("Airport Selected in Table");
              }
            }
          });
    }

  /**
  *  Sets up the listeners that respond to user actions, like button presses
  **/
  public void setupListeners() {
    // Load Sample button listener
    m_loadButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // Action to be performed when Load Samples button is pressed
        if (m_table.getSelectedRow() > 0)
          processEvent("Load Sample Files");
        else
          putStatus("Please choose a row from the JTable first");
      }
    });

    // Add Suggestions button listener
    m_addButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // Action to be performed when Load Samples button is pressed
        if (m_table.getSelectedRow() > 0)
          processEvent("Add Suggestions");
        else
          putStatus("Please choose a row from the JTable first");
      }
    });

    // Exit button listener
    m_exitButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // Action to be performed when Load Samples button is pressed
        m_parent.dispatchEvent("EXIT");
      }
    });

    this.addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        m_parent.dispatchEvent("EXIT");
      }
    });
  }

  /**
  * Status field message display handlers
  **/
      public void clearStatus() {         // Clears status field
        m_statusField.setText("");
        m_statusField.setScrollOffset(0);
       }
      public void putStatus(String p_status) { // Puts p_status on status field
        m_statusField.setText(p_status);
        m_statusField.setScrollOffset(0);
      }
      public void appendStatus(String p_status) {  //Appends p_status to status
                                                   //Field
        String l_status = m_statusField.getText();
        m_statusField.setText(l_status+p_status);
        m_statusField.setScrollOffset(0);
      }


  /**
  * Initialize all the GUI components and the layout
  **/
  public void jbInit() {
    Border emptyBorder5 = new EmptyBorder(5,5,5,5);
    Border bevelBorder10 = new CompoundBorder(
               new BevelBorder(SoftBevelBorder.RAISED),emptyBorder5);
    Border etchedBorder10 = new CompoundBorder(new EtchedBorder(), emptyBorder5);
    Border titleBorder = new TitledBorder(bevelBorder10,"LOB Details",
                                   TitledBorder.LEFT,TitledBorder.BELOW_TOP);

    m_scrollPane = new JScrollPane();
    m_imagePane = new JScrollPane();
    m_suggestionPane = new JScrollPane();

    m_loadButton = new JButton("LOAD SAMPLE FILES");
    m_addButton = new JButton("ADD SUGGESTIONS");
    m_exitButton = new JButton("EXIT");

    this.setTitle("Illustrating LOBs using JDBC");
    this.getContentPane().setLayout(m_gridBagLayout);
    this.setSize(new Dimension(598, 592));
    m_tabPane.setFont(new Font("Dialog", 1, 14));
    m_loadButton.setFont(new Font("Dialog", 1, 16));
    m_loadButton.setToolTipText("Press to load Sample files");
    m_loadButton.setBorder(bevelBorder10);
    m_loadButton.setEnabled(false);
    m_addButton.setFont(new Font("Dialog", 1, 16));
    m_addButton.setBorder(bevelBorder10);
    m_addButton.setEnabled(false);
    m_addButton.setToolTipText("Press to add suggestions");
    m_exitButton.setFont(new Font("Dialog", 1, 16));
    m_exitButton.setBorder(bevelBorder10);
    m_exitButton.setToolTipText("Exit Application");
    m_statusField.setBackground(Color.lightGray);
    m_statusField.setFont(new Font("Monospaced", 0, 12));
    m_statusField.setEditable(false);
    m_lobPanel.setLayout(m_borderLayout);
    m_lobPanel.setBorder(titleBorder);
    m_imagePane.setBackground(Color.lightGray);
    m_imagePane.setBorder(etchedBorder10);
    m_imagePane.setViewportView(m_imageArea);
    this.getContentPane().add(m_lobPanel, new GridBagConstraints2(0, 1, 4, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(8, 9, 0, 6), 548, 40));
    m_lobPanel.add(m_tabPane, BorderLayout.CENTER);
    m_tabPane.addTab("Airport Map", m_imagePane);
    m_imagePane.getViewport().add(m_imageArea, null);
    m_tabPane.addTab("Suggestion Box", m_suggestionPane);
    m_suggestionPane.getViewport().add(m_sugArea, null);
    this.getContentPane().add(m_scrollPane, new GridBagConstraints2(0, 0, 4, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.VERTICAL, new Insets(7, 41, 0, 44), 424, -324));
    m_scrollPane.getViewport().add(m_table, null);
    this.getContentPane().add(m_exitButton, new GridBagConstraints2(2, 2, 2, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(11, 20, 0, 17), 39, 6));
    this.getContentPane().add(m_addButton, new GridBagConstraints2(1, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 7));
    this.getContentPane().add(m_loadButton, new GridBagConstraints2(0, 2, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 67, 0, 0), 0, 7));

    this.getContentPane().add(m_statusField, new GridBagConstraints2(0, 3, 5, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(10, 0, 4, 0), 586, 47));
  }
}
