/**
 * @author  rsundara.in
 * @version 1.0
 *
 * Development Environment        :  JDeveloper 2.0
 * Name of the Application        :  StreamFrame.java
 * Creation/Modification History  :
 *
 * rsundara.in       07-Jan-1999     Created
 *
 */

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;

/** Use the following import statements with SWING version 1.0.2   **/
import com.sun.java.swing.*;

/** Use the following import statements with SWING version 1.1   **/
//import javax.swing.*;

public class StreamFrame extends JFrame {

  StreamSample m_parent;
  static String s_fileName = "information.txt";

  //Icon object whichholds a image files
  ImageIcon m_radio, m_radioPressed, m_radioSelected;

  TextField m_statusbar = new TextField();           //Status bar
  JPanel m_panel1, m_panel2, m_panel3, m_radioPanel; //Jpanels
  JLabel m_label1, m_label2, m_label3;               //Labels used in the UI
  TextArea m_textarea1, m_textarea2;  //TextArea for displaying the output

  //Buttons for specifying the operations to be performed
  JRadioButton m_ascii, m_unicode, m_binary, m_exit;

  //ButtonGroup for holding the Radio Buttons
  ButtonGroup m_group=new ButtonGroup();

  //Layout manager which layout the components
  GridBagLayout m_gridBagLayout1, m_gridBagLayout2, m_gridBagLayout3;
  BorderLayout m_borderLayout1   = new BorderLayout();

 /**
  * Constructor. Initializes the JTable and other GUI components.
 **/
  public StreamFrame(StreamSample p_parent){
    try{
      m_parent=p_parent;
      jbInit();         //GUI components initializations
      setupListeners(); //Setting up event listeners and actions
    }catch (Exception ex){ //Trap General Errors
      putStatus("Error...!!! "+ ex.toString());
     }
  }

  /**
  * Retrieve the File contents and display the contents in the GUI TextArea
  * p_numChars : Number of characters after which to wrap
  **/
  public String displayText(int p_numChars) {
    m_textarea2.setText("");
    try {
      FileInputStream l_fis=new FileInputStream(new File(s_fileName)); // Open file
      int l_dataread=0;
      StringBuffer l_strbuf=new StringBuffer();
      int l_cnt = 0;

      // Loop through the file, save in stringbuffer and also write to GUI
      while((l_dataread = l_fis.read()) != -1) {
        Character l_read = new Character((char)l_dataread);
        if (!l_read.toString().equals("\n")) {
          l_strbuf.append(l_read.toString());
          m_textarea2.append(l_read.toString()+"|");

          // Wrap after p_numChars
          l_cnt++;
          if (l_cnt == p_numChars) {
            l_cnt = 0;
            m_textarea2.append("\n");
          }
        }
      }
      l_fis.close();  //close the fileinputstream
      return new String(l_strbuf); // Return the file text
    } catch (IOException ex) {
      return null;
    }
  }

  /**
  * Pad the input integer with zeros (for 3chars) and print in GUI
  * along with the demarcator
  **/
  public void padAndAppend(int p_str,String p_demarcator) {
    String l_str = new Integer(p_str).toString();
    if (l_str.length() == 1)
      m_textarea1.append("00"+l_str+p_demarcator);
    else if (l_str.length() == 2)
      m_textarea1.append("0"+l_str+p_demarcator);
    else
      m_textarea1.append(l_str+p_demarcator);
  }

 /**
  * Sets up the listeners that respond to user actions, including button
  * press
 **/

 public void setupListeners(){
   m_ascii.addActionListener(new java.awt.event.ActionListener(){
     public void actionPerformed(ActionEvent e){
       try{
          m_label1.setForeground(Color.black);
          m_label1.setText("ASCII Data Stream");
          putStatus("Reading Ascii data stream...");
          Thread.sleep(1000);
          m_textarea1.setText("");
          m_parent.dispatchEvent("ASCII");
       }catch(InterruptedException ex){
           putStatus(ex.toString());
        }
     }
    });

   m_unicode.addActionListener(new java.awt.event.ActionListener() {
     public void actionPerformed(ActionEvent e) {
       try{
          m_label1.setForeground(Color.black);
          m_label1.setText("UNICODE Data Stream");
          putStatus("Reading Unicode data stream...");
          Thread.sleep(1000);
          m_textarea1.setText("");
          m_parent.dispatchEvent("UNICODE");
       }catch(InterruptedException ex){
           putStatus(ex.toString());
        }
     }
   });
   m_binary.addActionListener(new java.awt.event.ActionListener() {
     public void actionPerformed(ActionEvent e) {
       try{
          m_label1.setForeground(Color.black);
          m_label1.setText("BINARY Data Stream");
          putStatus("Reading Binary data stream...");
          Thread.sleep(1000);
          m_textarea1.setText("");
          m_parent.dispatchEvent("BINARY");
       }catch(InterruptedException ex){  //Trap Interrupted error of thread
          putStatus(ex.toString());
        }
     }
  });

   m_exit.addActionListener(new java.awt.event.ActionListener() {
     public void actionPerformed(ActionEvent e){
       if(e.getActionCommand() =="EXIT"){
          m_parent.dispatchEvent("EXIT");
        }
     }
   });
 }

 /**
  *  Status field message display handlers
 **/

 public void appendStatus(String p_status){
    String l_status=m_statusbar.getText();
    m_statusbar.setText(l_status + ":" + p_status);
 }
 public void putStatus(String p_message){
    m_statusbar.setText(p_message);
 }
 public void clearStatus(){
    m_statusbar.setText(" ");
 }

 /**
  *  Initialize all the GUI components and the layout
 **/

  public void jbInit() throws Exception {
    m_gridBagLayout1 = new GridBagLayout();
    m_gridBagLayout2 = new GridBagLayout();
    m_gridBagLayout3 = new GridBagLayout();
    m_panel1 = new JPanel();
    m_panel2 = new JPanel();
    m_panel3 = new JPanel();
    m_radioPanel = new JPanel();
    m_label1 = new JLabel();
    m_label2 = new JLabel();
    m_label3 = new JLabel();
    m_radio=new ImageIcon("radio.gif");
    m_radioPressed=new ImageIcon("radioPressed.gif");
    m_radioSelected=new ImageIcon("radioSelected.gif");
    m_ascii=new JRadioButton("Ascii",m_radio);
    m_ascii.setPressedIcon(m_radioPressed);
    m_ascii.setRolloverIcon(m_radioPressed);
    m_ascii.setSelectedIcon(m_radioSelected);
    m_textarea1 = new TextArea();
    m_textarea2 = new TextArea();
    m_statusbar.setFont(new Font("Monospaced", 0, 16));
    m_group.add(m_ascii);
    m_binary=new JRadioButton("Binary",m_radio);
    m_binary.setPressedIcon(m_radioPressed);
    m_binary.setRolloverIcon(m_radioPressed);
    m_binary.setSelectedIcon(m_radioSelected);
    m_group.add(m_binary);
    m_unicode=new JRadioButton("Unicode",m_radio);
    m_unicode.setPressedIcon(m_radioPressed);
    m_unicode.setRolloverIcon(m_radioPressed);
    m_unicode.setSelectedIcon(m_radioSelected);
    m_group.add(m_unicode);
    m_exit=new JRadioButton("Exit",m_radio);
    m_exit.setRolloverIcon(m_radioPressed);
    m_group.add(m_exit);
    this.getContentPane().setLayout(m_borderLayout1);
    this.setTitle("Illustrating streams through JDBC");
    this.setSize(new Dimension(681, 507));
    this.addWindowListener(new java.awt.event.WindowAdapter() {
    public void windowClosing(WindowEvent e) {
        m_parent.dispatchEvent("EXIT");;
      }
    });
    this.setVisible(true);
    m_panel1.setForeground(Color.black);
    m_panel1.setBackground(Color.lightGray);
    m_panel1.setFont(new Font("Monospaced", 0, 12));
    m_panel1.setLayout(m_gridBagLayout1);
    m_panel2.setBackground(Color.lightGray);
    m_panel2.setFont(new Font("Monospaced", 0, 12));
    m_panel2.setLayout(m_gridBagLayout2);
    m_panel3.setBackground(Color.lightGray);
    m_textarea1.setFont(new Font("Monospaced", 0, 14));
    m_textarea1.setEditable(false);
    m_textarea1.setBackground(Color.white);
    m_textarea1.setForeground(Color.black);
    m_textarea2.setFont(new Font("Monospaced", 0, 14));
    m_textarea2.setEditable(false);
    m_textarea2.setBackground(Color.white);
    m_textarea2.setForeground(Color.black);
    m_panel3.setLayout(m_gridBagLayout3);
    m_label1.setText("");
    m_label1.setFont(new Font("Dialog", 1, 16));
    m_label2.setForeground(Color.black);
    m_label2.setFont(new Font("Dialog", 1, 16));
    m_label2.setText("TEXT");
    m_label3.setForeground(Color.black);
    m_label3.setFont(new Font("Dialog", 1, 16));
    m_label3.setText("Stream Type     :     ");
    m_ascii.setToolTipText("Click here for ascii format...");
    m_binary.setToolTipText("Click here for binary format...");
    m_exit.setToolTipText("Click here to exit...");
    m_exit.setActionCommand("EXIT");
    m_statusbar.setEditable(false);
    m_unicode.setToolTipText("Click here for unicode format...");
    this.getContentPane().add(m_panel1, BorderLayout.NORTH);
    m_panel1.add(m_label1, new GridBagConstraints2(0, 0, 2, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(10, 14, 0, 0), 225, 19));
    m_panel1.add(m_textarea1, new GridBagConstraints2(0, 1, 2, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.VERTICAL, new Insets(0, 5, 8, 0), 411, 141));
    m_panel1.add(m_label2, new GridBagConstraints2(2, 0, 1, 1, 0.0, 0.0
    ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(10, 0, 0, 0), 78, 8));
    m_panel1.add(m_textarea2, new GridBagConstraints2(2, 1, 2, 1, 1.0, 1.0
            ,GridBagConstraints.CENTER, GridBagConstraints.VERTICAL, new Insets(0, 4, 7, 10), 121, 142));
    m_radioPanel.add(m_label3);
    m_radioPanel.add(m_ascii);
    m_radioPanel.add(m_unicode);
    m_radioPanel.add(m_binary);
    m_radioPanel.add(m_exit);
    this.getContentPane().add(m_radioPanel, BorderLayout.CENTER);

    this.getContentPane().add(m_panel3, BorderLayout.SOUTH);
    m_panel3.add(m_statusbar, new GridBagConstraints2(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1, 3, 4, 0), 556, 11));
    this.setVisible(true);

 }

}
